==> gs-dbs-client-0.1.1.tar.gz: gs-dbs-client-0.1.1/client/gsdbs.py <==
import sys
import asyncio

import json
import yaml
import requests
from gql import gql, Client
from gql.transport.aiohttp import AIOHTTPTransport
import pandas as pd


class GSDBS:
    def __init__(self):
        self.version = "0.1.1"
        self.credentials = None
        self.accessToken = None
        self.dtablename = None
        self.superdtablename = None
        self.sriBuildInfo = list()
        self.data = pd.DataFrame()
        self.statement = None
        self.readCredentials()
        self.getToken()
        return

    def readCredentials(self):
        try:
            print(f'GS-DBMS Python client version: {self.version}')
            f = open("gscred.yml", "r")
            cred = f.read()
            f.close()
            self.credentials = yaml.safe_load(cred)['cred']

            if (('graphqlapiurl' not in self.credentials.keys())
                    or ('oauthurl' not in self.credentials.keys())
                    or ('apiurl' not in self.credentials.keys())
                    or ('user' not in self.credentials.keys())
                    or ('password' not in self.credentials.keys())
                    or ('client_id' not in self.credentials.keys())
                    or ('client_secret' not in self.credentials.keys())):
                print('Error in credentials')
                return
        except:
            print('Missing Credentials')
        return

    def getToken(self):
        data = {'grant_type': 'password', 'username': self.credentials['user'],
                'password': self.credentials['password']}
        access_token_response = requests.post(self.credentials['oauthurl'], data=data, verify=True,
                                              allow_redirects=False,
                                              auth=(self.credentials['client_id'], self.credentials['client_secret']))
        self.accessToken = json.loads(access_token_response.text)
        return

    def setDTableName(self, dTableName, superDTableName="DTABLE"):
        if dTableName is None or superDTableName is None:
            print("error: missing (super)dtablename data")
            self.dtablename = None
            self.superdtablename = None
            return -1;
        if (not (isinstance(dTableName, str) and isinstance(superDTableName, str))):
            print("error: missing (super)dtablename datatype \"str\"")
            self.dtablename = None
            self.superdtablename = None
            return -2;

        self.dtablename = dTableName
        self.superdtablename = superDTableName
        return 0

    def clearDTableName(self):
        self.dtablename = None
        self.superdtablename = None
        return 0

    def checkDTableName(self):
        if self.dtablename is None or self.superdtablename is None:
            print("error: missing (super)dtablename datatype \"str\"")
            return -1
        return 0

    def setSriBuildInfo(self, dataLinkList):
        if (dataLinkList is None):
            print("error: missing datalink")
            self.sriBuildInfo = None
            return -1;
        if not isinstance(dataLinkList, list):
            print("error: missing datatype \"list\"")
            self.sriBuildInfo = None
            return -2
        if (len(dataLinkList) == 0):
            print("error: missing data")
            self.sriBuildInfo = None
            return -3
        self.sriBuildInfo = dataLinkList
        return 0

    def getSriBuildInfo(self):
        buidInfo = str()
        for dl in self.sriBuildInfo:
            if len(buidInfo) == 0:
                buidInfo += f'${{{dl}}}'
            else:
                buidInfo += f'-${{{dl}}}'
        return buidInfo

    def clearSriBuildInfo(self):
        self.sriBuildInfo = None
        return 0

    def checkSriBuildInfo(self):
        if (self.sriBuildInfo is None):
            print("error: missing sri build info")
            return -1
        if (self.data is None):
            print("error: missing data")
            return -2
        for dl in self.sriBuildInfo:
            if dl not in self.data.columns:
                print("error: datalink " + dl + "not included in dataframe")
                return -3
        return 0

    def setData(self, data):
        if data is None:
            self.data = None
            print("error: missing data")
            return -1
        if not isinstance(data, pd.DataFrame):
            self.data == None
            print("error: missing datatype \"dataframe\"")
            return -2
        if len(data.index) == 0:
            self.data = None
            print("error: missing data rows")
            return -3
        self.data = data
        return 0

    def getGSDBSType(self, pyType):
        if pyType == "object":
            return "STRING"
        elif pyType == "int32":
            return "INTEGER"
        elif pyType == "int64":
            return "INTEGER"
        elif pyType == "float32":
            return "FLOAT"
        elif pyType == "float64":
            return "FLOAT"
        elif pyType == "bool":
            return "BOOLEAN"
        else:
            return None

    def getDataSchema(self):
        s = str()
        for col in self.data:
            gsDbsType = self.getGSDBSType(str(self.data.dtypes[col]))
            if gsDbsType == None:
                return -1
            s += f"""\t\t\t{{alias: "{col}", locale: DE, superPropertyURI: DYNAMIC_DATALINK, DataType: {gsDbsType}}},\n"""
        if len(s) > 0:
            s = s[:-2]
        return s

    def getData(self):
        # ["webcam2", "2020", "1337","10", "20", "100", "200", "0.5"]
        s = str()

        s += f"\t\t\t["
        for col in self.data:
            s += f""""{col}", """
        s = s[:-2]
        if len(self.data) > 0:
            s += "], \n"
        else:
            s += "]\n"

        for i in range(len(self.data)):
            s += f"\t\t\t["
            for col in self.data:
                s += f""""{str(self.data.loc[i, col])}", """
            s = s[:-2]
            if i < (len(self.data) - 1):
                s += "], \n"
            else:
                s += "]\n"
        return s

    def clearData(self):
        self.data = None
        return 0

    def checkData(self):
        if (self.data is None):
            print("error: missing data")
            return -1
        return 0

    def generateStatement(self):
        i = 1
        self.statement = \
            f"""mutation {{\n\taddDTable(dtablename: "{self.dtablename}", \n\t\tsuperDTable: [{self.superdtablename}], \n\t\tsriBuildInfo: "{self.getSriBuildInfo()}", \n\t\tdataLinks: [\n{self.getDataSchema()}\n\t\t], \n\t\tdata: [\n{self.getData()}\t\t]\n\t)\n}}"""
        return 0

    # ---------
    async def asyncSchemaCheck(self):
        # check mutation content
        if self.checkDTableName() != 0:
            return -1
        if self.checkData() != 0:
            return -2
        if self.checkSriBuildInfo() != 0:
            return -3

        # self.dtablename = "auf1"
        iQuery = await self.asyncSchemaQuery(self.dtablename)
        if iQuery == None:
            return -4

        if iQuery['__type'] == None:
            return 0  # dtable doesn't exist, do what you want

        schema = dict(iQuery['__type'])
        schemaName = schema["name"]
        schemaLinks = schema["fields"]
        return 0

    def schemaCheck(self):
        # check mutation content
        if self.checkDTableName() != 0:
            return -1
        if self.checkData() != 0:
            return -2
        if self.checkSriBuildInfo() != 0:
            return -3

        # self.dtablename = "auf1"
        iQuery = self.schemaQuery(self.dtablename)
        if iQuery == None:
            return -4

        if iQuery['__type'] == None:
            return 0  # dtable doesn't exist, do what you want

        schema = dict(iQuery['__type'])
        schemaName = schema["name"]
        schemaLinks = schema["fields"]
        return 0

    # ---------
    async def asyncAddDObject(self, dtablename, sribuildinfo, dataframe, superdtablename="DTABLE", schemaCheck=True):
        self.superdtablename = superdtablename
        rc = self.setDTableName(dtablename)
        if rc != 0: return rc

        rc = self.setSriBuildInfo(sribuildinfo)
        if rc != 0: return rc

        rc = self.setData(dataframe)
        if rc != 0: return rc

        if schemaCheck == True:
            rc = await self.asyncSchemaCheck()
            if rc != 0: return rc

        rc = self.generateStatement()
        if rc != 0: return rc

        rc = await self.asyncExecuteStatement()
        return rc

    def addDObject(self, dtablename, sribuildinfo, dataframe, superdtablename="DTABLE", schemaCheck=True):
        self.superdtablename = superdtablename
        rc = self.setDTableName(dtablename)
        if rc != 0: return rc

        rc = self.setSriBuildInfo(sribuildinfo)
        if rc != 0: return rc

        rc = self.setData(dataframe)
        if rc != 0: return rc

        if schemaCheck == True:
            rc = self.schemaCheck()
            if rc != 0: return rc

        rc = self.generateStatement()
        if rc != 0: return rc

        rc = self.executeStatement()
        return rc

    # ---------
    async def asyncSchemaQuery(self, dtablename):
        statement = "{ __type(name: \"" + "gun" + "\") {name fields { name type { name kind } } } }"
        result = await self.asyncExecuteStatement(statement)
        return result

    def schemaQuery(self, dtablename):
        statement = "{ __type(name: \"" + "gun" + "\") {name fields { name type { name kind } } } }"
        return self.executeStatement(statement)

    # ---------
    async def asyncExecuteStatement(self, statement=None):
        if statement == None:
            if self.statement == None:
                return -1
            query = gql(self.statement)
        else:
            query = gql(statement)

        api_call_headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
        client = Client(transport=AIOHTTPTransport(url="https://glass-sphere-ai.de/gsgraphql",
                                                   headers=api_call_headers),
                        fetch_schema_from_transport=False)
        result = await client.execute_async(query)
        return result

    def executeStatement(self, statement=None):
        if statement == None:
            if self.statement == None:
                return -1
            query = gql(self.statement)
        else:
            query = gql(statement)

        api_call_headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
        client = Client(transport=AIOHTTPTransport(url="https://glass-sphere-ai.de/gsgraphql",
                                                   headers=api_call_headers),
                        fetch_schema_from_transport=False)
        result = client.execute(query)
        return result

==> gs-dbs-client-0.1.3.tar.gz: gs-dbs-client-0.1.3/client/gsdbs.py <==
import sys
import asyncio

import json
import yaml
import requests
from gql import gql, Client
from gql.transport.aiohttp import AIOHTTPTransport
import pandas as pd


class GSDBS:
    def __init__(self):
        self.version = "0.1.3"
        self.credentials = None
        self.accessToken = None
        self.dtablename = None
        self.superdtablename = None
        self.sriBuildInfo = list()
        self.data = pd.DataFrame()
        self.statement = None
        self.readCredentials()
        self.getToken()
        self.CGSResponseCode = {}
        return

    def readCredentials(self):
        try:
            print(f'GS-DBMS Python client version: {self.version}')
            f = open("gscred.yml", "r")
            cred = f.read()
            f.close()
            self.credentials = yaml.safe_load(cred)['cred']

            if (('graphqlapiurl' not in self.credentials.keys())
                    or ('oauthurl' not in self.credentials.keys())
                    or ('apiurl' not in self.credentials.keys())
                    or ('user' not in self.credentials.keys())
                    or ('password' not in self.credentials.keys())
                    or ('client_id' not in self.credentials.keys())
                    or ('client_secret' not in self.credentials.keys())):
                print('Error in credentials')
                return
        except:
            print('Missing Credentials')
        return

    def getToken(self):
        data = {'grant_type': 'password', 'username': self.credentials['user'],
                'password': self.credentials['password']}
        access_token_response = requests.post(self.credentials['oauthurl'], data=data, verify=True,
                                              allow_redirects=False,
                                              auth=(self.credentials['client_id'], self.credentials['client_secret']))
        self.accessToken = json.loads(access_token_response.text)
        return

    def setupCGSResponseCodes(self):
        self.CGSResponseCode[0]  = 'SUCCESS'
        self.CGSResponseCode[-1] = 'MISSING (SUPER) DTABLE NAME'
        self.CGSResponseCode[-2] = 'MISSING DATA LINK LIST'
        self.CGSResponseCode[-3] = 'MISSING SRI BUILD INFO'
        self.CGSResponseCode[-4] = 'MISSING DATA'
        self.CGSResponseCode[-5] = 'MISSING DATALINK FOR SRI BUILD INFO in DATAFRAME'
        self.CGSResponseCode[-6] = 'QUERY RESULT IS NULL'
        self.CGSResponseCode[-7] = 'DATAFRAME DATATYPE DOESN\'T MATCHES DTABEL DATATYPE'
        self.CGSResponseCode[-8] = 'EXECUTION FAILED'
        self.CGSResponseCode[-9] = 'MISSING STATEMENT TO EXECUTE'

    def setDTableName(self, dTableName, superDTableName="DTABLE"):
        if dTableName is None or superDTableName is None:
            self.dtablename = None
            self.superdtablename = None
            return -1;
        if (not (isinstance(dTableName, str) and isinstance(superDTableName, str))):
            self.dtablename = None
            self.superdtablename = None
            return -1;
        self.dtablename = dTableName
        self.superdtablename = superDTableName
        return 0

    def clearDTableName(self):
        self.dtablename = None
        self.superdtablename = None
        return 0

    def checkDTableName(self):
        if self.dtablename is None or self.superdtablename is None:
            return -1
        return 0

    def setSriBuildInfo(self, dataLinkList):
        if (dataLinkList is None):
            self.sriBuildInfo = None
            return -2;
        if not isinstance(dataLinkList, list):
            self.sriBuildInfo = None
            return -2
        if (len(dataLinkList) == 0):
            self.sriBuildInfo = None
            return -2
        self.sriBuildInfo = dataLinkList
        return 0

    def getSriBuildInfo(self):
        buidInfo = str()
        for dl in self.sriBuildInfo:
            if len(buidInfo) == 0:
                buidInfo += f'${{{dl}}}'
            else:
                buidInfo += f'-${{{dl}}}'
        return buidInfo

    def clearSriBuildInfo(self):
        self.sriBuildInfo = None
        return 0

    def checkSriBuildInfo(self):
        if (self.sriBuildInfo is None):
            return -3
        if (self.data is None):
            return -4
        for dl in self.sriBuildInfo:
            if dl not in self.data.columns:
                return -5
        return 0

    def setData(self, data):
        if data is None:
            self.data = None
            return -4
        if not isinstance(data, pd.DataFrame):
            self.data == None
            return -4
        if len(data.index) == 0:
            self.data = None
            return -4
        self.data = data
        return 0

    def getGSDBSType(self, pyType):
        if pyType == "object":
            return "string"
        elif pyType == "int32":
            return "integer"
        elif pyType == "int64":
            return "integer"
        elif pyType == "float32":
            return "float"
        elif pyType == "float64":
            return "float"
        elif pyType == "bool":
            return "boolean"
        else:
            return None

    def getDataSchema(self):
        s = str()
        for col in self.data:
            gsDbsType = self.getGSDBSType(str(self.data.dtypes[col]))
            if gsDbsType == None:
                return -1
            s += f"""\t\t\t{{alias: "{col}", locale: DE, superPropertyURI: DYNAMIC_DATALINK, DataType: {gsDbsType}}},\n"""
        if len(s) > 0:
            s = s[:-2]
        return s

    def getData(self):
        # ["webcam2", "2020", "1337","10", "20", "100", "200", "0.5"]
        s = str()

        s += f"\t\t\t["
        for col in self.data:
            s += f""""{col}", """
        s = s[:-2]
        if len(self.data) > 0:
            s += "], \n"
        else:
            s += "]\n"

        for i in range(len(self.data)):
            s += f"\t\t\t["
            for col in self.data:
                s += f""""{str(self.data.loc[i, col])}", """
            s = s[:-2]
            if i < (len(self.data) - 1):
                s += "], \n"
            else:
                s += "]\n"
        return s

    def clearData(self):
        self.data = None
        return 0

    def checkData(self):
        if (self.data is None):
            return -4
        return 0

    def generateMutationStatement(self):
        self.statement = \
            f"""mutation {{\n\taddDTable(dtablename: "{self.dtablename}", \n\t\tsuperDTable: [{self.superdtablename}], \n\t\tsriBuildInfo: "{self.getSriBuildInfo()}", \n\t\tdataLinks: [\n{self.getDataSchema()}\n\t\t], \n\t\tdata: [\n{self.getData()}\t\t]\n\t)\n}}"""
        return

    # ---------
    async def asyncSchemaCheck(self):
        # check mutation content
        rc = self.checkDTableName()
        if rc != 0: return rc
        rc = self.checkData()
        if rc != 0: return rc
        rc = self.checkSriBuildInfo()
        if rc != 0: return rc

        iQuery = await self.asyncSchemaQuery(self.dtablename)
        if iQuery == None: return -6

        if iQuery['__type'] == None: return 0  # dtable doesn't exist, do what you want
        rc = self.schemaLinkCheck(iQuery)
        return rc

    def schemaCheck(self):
        # check mutation content
        rc = self.checkDTableName()
        if rc != 0: return rc
        rc = self.checkData()
        if rc != 0: return rc
        rc = self.checkSriBuildInfo()
        if rc != 0: return rc

        iQuery = self.schemaQuery(self.dtablename)
        if iQuery == None: return -6

        if iQuery['__type'] == None: return 0  # dtable doesn't exist, do what you want
        rc = self.schemaLinkCheck(iQuery)
        return rc

    def schemaLinkCheck(self, iQuery):
        schema = dict(iQuery['__type'])
        schemaName = schema["name"]
        schemaLinks = schema["fields"]

        for col in self.data:
            dfType = self.getGSDBSType(str(self.data.dtypes[col]))
            linkType = self.getLinkType(schemaLinks, col)
            if dfType.lower() != linkType.lower():
                print('ERROR: dataframe datatype ', dfType, ' doesnt\' matches dTable datatype', linkType)
                return -7
        return 0

    def getLinkType(self, schemaLinks, linkName):
        if linkName == None:
            return -1
        for link in schemaLinks:
            if link['name'] == linkName:
                return link['type']['name']
        return None

    # ---------
    async def asyncAddDObject(self, dtablename, sribuildinfo, dataframe, superdtablename="DTABLE", schemaCheck=True):
        self.superdtablename = superdtablename
        rc = self.setDTableName(dtablename)
        if rc != 0: return rc
        rc = self.setSriBuildInfo(sribuildinfo)
        if rc != 0: return rc
        rc = self.setData(dataframe)
        if rc != 0: return rc

        if schemaCheck == True:
            rc = await self.asyncSchemaCheck()
            if rc != 0: return rc

        self.generateMutationStatement()
        try:
            rc = await self.asyncExecuteStatement()
        except:
            return None
        return rc

    def addDObject(self, dtablename, sribuildinfo, dataframe, superdtablename="DTABLE", schemaCheck=True):
        self.superdtablename = superdtablename
        rc = self.setDTableName(dtablename)
        if rc != 0: return rc
        rc = self.setSriBuildInfo(sribuildinfo)
        if rc != 0: return rc
        rc = self.setData(dataframe)
        if rc != 0: return rc

        if schemaCheck == True:
            rc = self.schemaCheck()
            if rc != 0: return rc

        self.generateMutationStatement()
        try:
            rc = self.executeStatement()
        except:
            return None
        return rc

    # ---------
    async def asyncSchemaQuery(self, dtablename):
        statement = "{ __type(name: \"" + "gun" + "\") {name fields { name type { name kind } } } }"
        try:
            result = await self.asyncExecuteStatement(statement)
        except:
            return None
        return result

    def schemaQuery(self, dtablename):
        statement = "{ __type(name: \"" + "gun" + "\") {name fields { name type { name kind } } } }"
        try:
            result = self.executeStatement(statement)
        except:
            return None
        return result

    # ---------
    async def asyncExecuteStatement(self, statement=None):
        if statement == None:
            if self.statement == None: return -9
            query = gql(self.statement)
        else:
            query = gql(statement)

        api_call_headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
        client = Client(transport=AIOHTTPTransport(url="https://glass-sphere-ai.de/gsgraphql",
                        headers=api_call_headers),
                        fetch_schema_from_transport=False)
        try:
            result = await client.execute_async(query)
        except:
            return None
        return result

    def executeStatement(self, statement=None):
        if statement == None:
            if self.statement == None: return -9
            query = gql(self.statement)
        else:
            query = gql(statement)

        api_call_headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
        client = Client(transport=AIOHTTPTransport(url="https://glass-sphere-ai.de/gsgraphql",
                        headers=api_call_headers),
                        fetch_schema_from_transport=False)
        try:
            result = client.execute(query)
        except:
            return None
        return result

    def asyncDropDTable(self, dtablename):
        statement = f"""mutation {{\n\tdropDTable(dtablename: {dtablename})\n}}"""
        result = self.AsyncExecuteStatement(statement)
        return result

    def dropDTable(self, dtablename):
        statement = f"""mutation {{\n\tdropDTable(dtablename: {dtablename})\n}}"""
        result = self.executeStatement(statement)
        return result
